import { Plugin } from 'ckeditor5/src/core';

import {
  clickOutsideHandler,
  ContextualBalloon,
  buttonView,
  addListToDropdown,
  ViewModel,
  ButtonView,
  createDropdown,
} from 'ckeditor5/src/ui';
import { Collection } from 'ckeditor5/src/utils';
import icon from '../../../../../icons/table-properties.svg';

// Refactored to share common code
import MainTableFormView from '../view/view';
import AdvancedTableFormView from '../view/advancedView';

import {
  getSelectionAffectedTable,
} from '@ckeditor/ckeditor5-table/src/utils/common';
import { getBalloonTablePositionData } from '../utils/positionContextualBalloon';

export default class TablesUi extends Plugin {
  static get requires() {
    return [ContextualBalloon];
  }

  /**
   * @inheritDoc
   */
  static get pluginName() {
    return 'Tables';
  }

  /**
   * @inheritDoc
   */
  init() {
    const editor = this.editor;

    this._balloon = editor.plugins.get(ContextualBalloon);
    this.formView = this._createFormView();

    editor.ui.componentFactory.add('Tables', (local) => {
      const buttonView = new ButtonView(local);
      buttonView.set({
        label: editor.t('Table properties'),
        withText: false,
        icon,
        tooltip: true,
      });
      buttonView.bind('isEnabled').to(editor, 'isReadOnly', isReadOnly => !isReadOnly);

      this.listenTo(buttonView, 'execute', (eventInfo) => {
        // Show the UI on button click.
        this._showUI();
      });

      return buttonView;
    });
  }

  _createFormView() {
    const editor = this.editor;
    const formView = new MainTableFormView(editor.locale, editor);

    this.listenTo(formView, 'submit', () => {
      const value = {
        row: formView.rowsInputView.fieldView.element.value,
        col: formView.columnsInputView.fieldView.element.value,
        header: formView.headerRowsViewValue,
        headerLabel: formView.headerRowsView,
        style: formView.classOptionsViewValue,
        styleLabel: formView.classOptionsView,
      };

      // Validation of value
      if (value.header === 'none' || !value.header) {
        window.alert('Please select header option');
        this.formView.headerRowsView.fieldView.focus();
      } else {
        editor.execute('PropertiesCommand', value);
        this._hideUI();
        this.formView.columnsInputView.fieldView.value = value.row;
        this.formView.rowsInputView.fieldView.value = value.col;
        this.formView.headerRowsView.fieldView.label = '';
      }

    });
    this.listenTo(formView, 'cancel', () => {
      const value = {
        row: formView.rowsInputView.fieldView.element.value,
        col: formView.columnsInputView.fieldView.element.value,
      };
      this.formView.columnsInputView.fieldView.value = value.row;
      this.formView.rowsInputView.fieldView.value = value.col;
      this._hideUI();
    });
    // Hide the form view when clicking outside the balloon.
    clickOutsideHandler({
      emitter: formView,
      activator: () => this._balloon.visibleView === formView,
      contextElements: [this._balloon.view.element],
      callback: () => this._hideUI(),
    });

    // Close the panel on esc key press when the form has focus.
    formView.keystrokes.set('Esc', (data, cancel) => {
      this._hideUI();
      cancel();
    });
    return formView;
  }

  // Hide the Form
  _hideUI() {
    const selection = this.editor.model.document.selection;
    const table = getSelectionAffectedTable(selection);
    if (!!table) {
      // do nothing
    } else {
      const value = {
        row: this.formView.rowsInputView.fieldView.element.value,
        col: this.formView.columnsInputView.fieldView.element.value,
      };
      this.formView.columnsInputView.fieldView.value = value.row;
      this.formView.rowsInputView.fieldView.value = value.col;
    }
    this._balloon.remove(this.formView);

    // Focus the editing view after closing the form view.
    this.editor.editing.view.focus();
  }

  _getBalloonPositionData() {
    const view = this.editor.editing.view;
    const viewDocument = view.document;

    let target = null;

    // Set a target position by converting view selection range to DOM.
    target = () => view.domConverter.viewRangeToDom(
        viewDocument.selection.getFirstRange(),
    );

    return {
      target,
    };
  }
  _setDefaultValues() {
    this.formView.columnsInputView.fieldView.value = '2';
    this.formView.rowsInputView.fieldView.value = '3';
  }
  _showUI() {
    const selection = this.editor.model.document.selection;
    const table = getSelectionAffectedTable(selection);

    const editor = this.editor;
    this._setDefaultValues();

    const style_classes = editor.config._config.tableStyles;
    console.info('The style options are', style_classes)

    const activeClass = 'ck-dropdown-selected';
    if (!!table) {
      // Populate fields for editing a table
      if (style_classes && style_classes.length > 0) {
        for (const tableClass of style_classes) {
          const id = tableClass.id;
          const label = tableClass.label;
          const machineName = table.getAttribute(id);

          if (machineName) {
            // Matching attribute selected
            this.formView.classOptionsView.fieldView.buttonView.label = label;
            this.formView.classOptionsViewValue = id;
          }
        }
      }


      // We ask for a different position for editing tables with helper function
      this._balloon.add({
        view: this.formView,
        position: getBalloonTablePositionData(this.editor),
      });

      // Disable fields here
      this.formView.rowsInputView.fieldView.element.disabled = true;
      this.formView.columnsInputView.fieldView.element.disabled = true;


      const first_row = table.getAttribute('headingRows');
      const first_col = table.getAttribute('headingColumns');

      this.formView.classOptionsView.fieldView.class = activeClass;
      this.formView.headerRowsView.fieldView.class = activeClass;

      if (first_row && first_col) {
        this.formView.headerRowsView.fieldView.buttonView.label = 'Both';
        this.formView.headerRowsViewValue = 'both';
      } else if (first_row) {
        this.formView.headerRowsView.fieldView.buttonView.label = 'First Row';
        this.formView.headerRowsViewValue = 'first_row';
      } else if (first_col) {
        this.formView.headerRowsView.fieldView.buttonView.label = 'First Column';
        this.formView.headerRowsViewValue = 'first_col';
      }
      const rows = Array.from(table.getChildren());
      const columns = Array.from(rows[0].getChildren());
      this.formView.columnsInputView.fieldView.value = columns.length;
      this.formView.rowsInputView.fieldView.value = rows.length;
    } else {
      // New Table inserted
      this._balloon.add({
        view: this.formView,
        position: this._getBalloonPositionData(),
      });

      this.formView.rowsInputView.fieldView.element.disabled = false;
      this.formView.columnsInputView.fieldView.element.disabled = false;
      this._setDefaultValues();
      this.formView.classOptionsView.fieldView.class = '';
      this.formView.headerRowsView.fieldView.class = '';

      this.formView.headerRowsView.fieldView.buttonView.label = 'Headers';
      this.formView.headerRowsViewValue = '';
    }

    this.formView.focus();
  }
}

