<?php
declare(strict_types=1);

namespace Drupal\ckeditor_styling_tables\Plugin\CKEditor5Plugin;

use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableTrait;
use Drupal\ckeditor5\Plugin\CKEditor5PluginDefault;
use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\EditorInterface;

/**
 * CKEditor 5 Styling Tables plugin.
 *
 * @CKEditor5Plugin(
 *   id = "ckeditor_styling_tables_tables",
 * ),
 * drupal = @DrupalAspectsOfCKEditor5Plugin(
 *   label = @Translation("Ckedtior 5 styling tables"),
 * )
 * @noinspection AnnotationMissingUseInspection
 */
class StylingTables extends CKEditor5PluginDefault implements CKEditor5PluginConfigurableInterface
{
  use CKEditor5PluginConfigurableTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration()
  {
    return [
      'table_styles' => [
        [
          'id' => 'table_styles',
          'classes' => '',
          'label' => 'Table Styles',
        ]
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $form['table_styles'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Add your classes to chose from'),
      '#description' => $this->t('Enter one style per line in the format: id|classes|label. For example: "defaultTable|usa-table|Default".'),
      '#default_value' => $this->configuration['table_styles'],
      '#rows' => 10,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    $table_styles = $form_state->getValue('table_styles');

    // Validate that each line has the correct format.
    $lines = explode("\n", $table_styles);
    $errors = [];

    foreach ($lines as $line_number => $line) {
      $line = trim($line);
      if (!empty($line)) {
        $parts = explode('|', $line);
        if (count($parts) < 3) {
          $errors[] = $this->t('Line @line: Each style must have an ID, classes, and a label, separated by "|" characters.', ['@line' => $line_number + 1]);
        }
      }
    }

    if (!empty($errors)) {
      $form_state->setError($form['table_styles'], implode('<br>', $errors));
    }
  }


  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    $this->configuration['table_styles'] = $form_state->getValue('table_styles');
  }

  /**
   * {@inheritdoc}
   */
  public function getDynamicPluginConfig(array $static_plugin_config, EditorInterface $editor): array
  {
    try {
      $table_styles = [];

      // Parse the configuration string into an array of style objects
      $lines = explode("\n", $this->configuration['table_styles'] ?? '');
      foreach ($lines as $line) {
        $line = trim($line);
        if (!empty($line)) {
          $parts = explode('|', $line, 3);
          if (count($parts) >= 3) {
            list($id, $classes, $label) = $parts;

            // Handle multiple classes by converting space-separated classes to an array
            if (strpos($classes, ' ') !== false) {
              $classes = explode(' ', $classes);
            }

            $table_styles[] = [
              'id' => $id,
              'classes' => $classes,
              'label' => $label,
            ];
          }
        }
      }

      // plugin configuration
      $static_plugin_config['tableStyles'] = $table_styles;

      return $static_plugin_config;
    } catch (\Exception $e) {
      // Return the static config as fallback
      return $static_plugin_config;
    }
  }

  /*
   *  Check if the existing configuration is null and initialize if needed
   */
  public function setConfiguration(array $configuration): void
  {
    if (empty($this->configuration)) {
      $this->configuration = [];
    } else {
      $this->configuration = $configuration;
    }

    // Now safely merge configurations
    //$this->configuration = $configuration + $this->configuration;
  }
}
